@@ -454,7 +454,7 @@ namespace std {
         typedef _BinBase<_Oper,valarray<_Tp>,_Dom> _Base;
         typedef typename _Base::value_type value_type;
 
-        _BinClos (const valarray<_Tp> __e1, const _Dom& __e2)
+        _BinClos (const valarray<_Tp>& __e1, const _Dom& __e2)
                 : _Base (__e1, __e2) {}
     };
 
@@ -556,41 +556,38 @@ namespace std {
     public:
         typedef typename _Dom::value_type value_type;
         
-        _GBase (const _Dom& __e, const gslice& __gs)
-                : _M_expr (__e), _M_size (__gs._M_index_size), 
-            _M_index (__gs._M_index) {}
+        _GBase (const _Dom& __e, const valarray<size_t>& __i)
+                : _M_expr (__e), _M_index(__i) {}
         value_type operator[] (size_t __i) const
-        { return _M_expr[_M_index._M_data[__i]]; }
-        size_t size () const { return _M_size; }
+        { return _M_expr[_M_index[__i]]; }
+        size_t size () const { return _M_index.size(); }
         
     private:
         const _Dom&	 _M_expr;
-        const size_t 	 _M_size;
-        const _Array<size_t> _M_index;
+        const valarray<size_t>& _M_index;
     };
     
     template<typename _Tp> class _GBase<_Array<_Tp> > {
     public:
         typedef _Tp value_type;
         
-        _GBase (_Array<_Tp> __a, const gslice& __gs)
-                : _M_array (__a), _M_size(__gs._M_index_size), 
-                    _M_index (__gs._M_index) {}
+        _GBase (_Array<_Tp> __a, const valarray<size_t>& __i)
+                : _M_array (__a), _M_index(__i) {}
         value_type operator[] (size_t __i) const
-        { return _M_array._M_data[_M_index._M_data[__i]]; }
-        size_t size () const { return _M_size; }
+        { return _M_array._M_data[_M_index[__i]]; }
+        size_t size () const { return _M_index.size(); }
         
     private:
         const _Array<_Tp>     _M_array;
-        const size_t         _M_size;
-        const _Array<size_t> _M_index;
+        const valarray<size_t>& _M_index;
     };
 
     template<class _Dom> struct _GClos<_Expr,_Dom> : _GBase<_Dom> {
         typedef _GBase<_Dom> _Base;
         typedef typename _Base::value_type value_type;
 
-        _GClos (const _Dom& __e, const gslice& __gs) : _Base (__e, __gs) {}
+        _GClos (const _Dom& __e, const valarray<size_t>& __i)
+                : _Base (__e, __i) {}
     };
 
     template<typename _Tp>
@@ -598,7 +595,8 @@ namespace std {
         typedef _GBase<_Array<_Tp> > _Base;
         typedef typename _Base::value_type value_type;
 
-        _GClos (_Array<_Tp> __a, const gslice& __gs) : _Base (__a, __gs) {}
+        _GClos (_Array<_Tp> __a, const valarray<size_t>& __i)
+                : _Base (__a, __i) {}
     };
 
     //
